from unittest.mock import MagicMock, patch

import pytest
from pydantic import ValidationError

from sporestack import api_client


def test__is_onion_url() -> None:
    onion_url = "http://spore64i5sofqlfz5gq2ju4msgzojjwifls7"
    onion_url += "rok2cti624zyq3fcelad.onion/v2/"
    assert api_client._is_onion_url(onion_url) is True
    # This is a good, unusual test.
    onion_url = "https://www.facebookcorewwwi.onion/"
    assert api_client._is_onion_url(onion_url) is True
    assert api_client._is_onion_url("http://domain.com") is False
    assert api_client._is_onion_url("domain.com") is False
    assert api_client._is_onion_url("http://onion.domain.com/.onion/") is False
    assert api_client._is_onion_url("http://me.me/file.onion/") is False
    assert api_client._is_onion_url("http://me.me/file.onion") is False


@patch("sporestack.api_client._api_request")
def test_launch(mock_api_request: MagicMock) -> None:
    with pytest.raises(ValidationError):
        api_client.launch(
            "dummymachineid",
            days=1,
            operating_system="freebsd-12",
            ssh_key="id-rsa...",
            flavor="aflavor",
            token="f" * 64,
        )


@patch("sporestack.api_client._api_request")
def test_topup(mock_api_request: MagicMock) -> None:
    with pytest.raises(ValidationError):
        api_client.topup("dummymachineid", token="f" * 64, days=1)


@patch("sporestack.api_client._api_request")
def test_start(mock_api_request: MagicMock) -> None:
    api_client.start("dummymachineid")
    mock_api_request.assert_called_once_with(
        "https://api.sporestack.com/server/dummymachineid/start", empty_post=True
    )


@patch("sporestack.api_client._api_request")
def test_stop(mock_api_request: MagicMock) -> None:
    api_client.stop("dummymachineid")
    mock_api_request.assert_called_once_with(
        "https://api.sporestack.com/server/dummymachineid/stop", empty_post=True
    )


@patch("sporestack.api_client._api_request")
def test_rebuild(mock_api_request: MagicMock) -> None:
    api_client.rebuild("dummymachineid")
    mock_api_request.assert_called_once_with(
        "https://api.sporestack.com/server/dummymachineid/rebuild", empty_post=True
    )


@patch("sporestack.api_client._api_request")
def test_info(mock_api_request: MagicMock) -> None:
    with pytest.raises(ValidationError):
        api_client.info("dummymachineid")
    mock_api_request.assert_called_once_with(
        "https://api.sporestack.com/server/dummymachineid/info"
    )


@patch("sporestack.api_client._api_request")
def test_delete(mock_api_request: MagicMock) -> None:
    api_client.delete("dummymachineid")
    mock_api_request.assert_called_once_with(
        "https://api.sporestack.com/server/dummymachineid/destroy", empty_post=True
    )


@patch("sporestack.api_client._api_request")
def test_token_balance(mock_api_request: MagicMock) -> None:
    with pytest.raises(ValidationError):
        api_client.token_balance("dummytoken")
    mock_api_request.assert_called_once_with(
        url="https://api.sporestack.com/token/dummytoken/balance"
    )



"""

SporeStack API request/response models

"""


from typing import List, Optional

from pydantic import BaseModel

from .models import Flavor, NetworkInterface, Payment


class TokenAdd:
    url = "/token/{token}/add"
    method = "POST"

    class Request(BaseModel):
        currency: str
        dollars: int
        affiliate_token: Optional[str] = None

    class Response(BaseModel):
        token: str
        payment: Payment


class TokenBalance:
    url = "/token/{token}/balance"
    method = "GET"

    class Response(BaseModel):
        token: str
        cents: int
        usd: str


class ServerLaunch:
    url = "/server/{machine_id}/launch"
    method = "POST"

    class Request(BaseModel):
        days: int
        flavor: str
        ssh_key: str
        operating_system: str
        region: Optional[str] = None
        """null is automatic, otherwise a string region slug."""
        token: str
        """Token to draw from when launching the server."""
        quote: bool = False
        """Don't launch, get a quote on how much it would cost"""
        affiliate_token: Optional[str] = None
        hostname: str = ""
        """Hostname to refer to your server by."""
        autorenew: bool = False
        """
        Automatically renew the server with the token used, keeping it at 1 week
        expiration.
        """

    class Response(BaseModel):
        payment: Payment
        """Deprecated, not needed when paying with token. Only used for quote."""
        expiration: int
        machine_id: str
        network_interfaces: List[NetworkInterface] = []
        """Deprecated, use ipv4/ipv6 from ServerInfo instead."""
        created_at: int = 0
        region: Optional[str] = None
        """Deprecated, use ServerInfo instead."""
        created: bool = False
        paid: bool = False
        """Deprecated, not needed when paying with token."""
        warning: Optional[str] = None
        flavor: str = ""
        """Deprecated, use ServerInfo instead."""


class ServerTopup:
    url = "/server/{machine_id}/topup"
    method = "POST"

    class Request(BaseModel):
        days: int
        token: str
        quote: bool = False
        affiliate_token: Optional[str] = None

    class Response(BaseModel):
        machine_id: str
        payment: Payment
        """Deprecated, not needed when paying with token."""
        expiration: int
        paid: bool = False
        """Deprecated, not needed when paying with token."""
        warning: Optional[str] = None


class ServerInfo:
    url = "/server/{machine_id}/info"
    method = "GET"

    class Response(BaseModel):
        created_at: int
        expiration: int
        running: bool
        machine_id: str
        token: str
        ipv4: str
        ipv6: str
        region: str
        flavor: Flavor
        deleted: bool
        network_interfaces: List[NetworkInterface]
        """Deprecated, use ipv4/ipv6 instead."""
        operating_system: str
        hostname: str
        autorenew: bool


class ServerStart:
    url = "/server/{machine_id}/start"
    method = "POST"


class ServerStop:
    url = "/server/{machine_id}/stop"
    method = "POST"


class ServerDelete:
    url = "/server/{machine_id}/delete"
    method = "POST"


class ServerDestroy:
    url = "/server/{machine_id}/destroy"
    method = "POST"


class ServerForget:
    url = "/server/{machine_id}/forget"
    method = "POST"


class ServerRebuild:
    url = "/server/{machine_id}/rebuild"
    method = "POST"


class ServerEnableAutorenew:
    url = "/server/{machine_id}/autorenew/enable"
    method = "POST"


class ServerDisableAutorenew:
    url = "/server/{machine_id}/autorenew/disable"
    method = "POST"


class ServersLaunchedFromToken:
    url = "/token/{token}/servers"
    method = "GET"

    class Response(BaseModel):
        servers: List[ServerInfo.Response]


class Flavors:
    url = "/flavors"
    method = "GET"

    class Response(BaseModel):
        flavors: dict[str, Flavor]


class OperatingSystems:
    url = "/operatingsystems"
    method = "GET"

    class Response(BaseModel):
        operating_systems: List[str]
